//@ts-check
"use strict";

/**
@type {{
  pixiRoot: PIXI.Container;

  timer: number;
  pages: PIXI.Container[];
}}
 */
let intro = null;

function registerIntroResources() {
  app.loader.add([
    "intro/outline.png",
    "intro/1.png",
    "intro/2.png",
    "intro/3.png",
    "intro/4.png",
  ]);
}

function enterIntro() {
  app.renderer.backgroundColor = 0xf0a867;

  const pixiRoot = new PIXI.Container();
  game.screenPixiRoot.addChild(pixiRoot);

  /** @type {PIXI.Container[]} */
  const pages = [];

  let imgSize = new PIXI.Point(209, 325);
  let width = 640;
  let height = 360;

  for (let j = 0; j < 3; j++) {
    let page = new PIXI.Container();
    for (let i = 0; i <= j; i++) {
      let icon = new PIXI.Sprite(tex(`intro/${i + 1}`));
      icon.anchor.set(0, 0);
      icon.x = width / 3 * (i + .5) - imgSize.x / 2;
      icon.y = height / 2 - imgSize.y / 2;
      icon.alpha = i == j ? 0.01 : 1;
      page.addChild(icon);
    }
    pages.push(page);
  }

  {
    let page = new PIXI.Container();
    let s = new PIXI.Sprite(tex(`intro/4`));
    s.anchor.set(.5, .5);
    s.alpha = 0.01;
    s.x = width / 2;
    s.y = height / 2;
    page.addChild(s);
    pages.push(page);
  }

  intro = {
    pixiRoot,
    timer: -1,
    pages,
  };

  document.addEventListener("keydown", onIntroKeyDown);
  document.addEventListener("pointerdown", onIntroPointerDown);
  app.ticker.add(animateIntro);
}

function leaveIntro(enter) {
  document.removeEventListener("pointerdown", onIntroPointerDown);
  document.removeEventListener("keydown", onIntroKeyDown);
  app.ticker.remove(animateIntro);
  intro.pixiRoot.destroy();

  intro = null;

  enter();
}

function onIntroKeyDown(event) {
  intro.timer = -1;
}

function onIntroPointerDown(event) {
  if (event.button !== 0) return;

  intro.timer = -1;
}

function animateIntro() {
  const dt = PIXI.Ticker.shared.elapsedMS / 1000;

  intro.timer -= dt;
  if (intro.timer < 0) {
    intro.timer = 3;
    let page = intro.pages.shift();
    if (!page) {
      leaveIntro(enterInGame);
      return;
    }

    intro.pixiRoot.removeChildren()
    intro.pixiRoot.addChild(page);
  }

  for (const parent of intro.pixiRoot.children) {
    // @ts-ignore
    if (parent.children) {
      // @ts-ignore
      for (const child of parent.children) {
        child.alpha = lerp(child.alpha, 1, 2 * dt);
      }
    }
  }
}